# -- coding: utf-8 --
# @Time : 2022/8/26
# @Author : ykk648
# @Project : https://github.com/ykk648/apstone
from .bbox_detector_mmdet import BboxDetectorBase
from .kp_detector_mmpose import KpDetectorBase
